@@ -1,6 +1,12 @@
 test_that("utility functios are functional!", {
   wrf    <- paste(system.file("extdata", package = "eva3dm"),"/wrfinput_d01", sep="")
 
+  # copy of the original file to a temporary folder
+  temp <- file.path(tempdir(),"test_utils")
+  dir.create(temp)
+  copy <- paste0(temp,"/wrfinput_d01")
+  file.copy(from = wrf,to = copy, overwrite = T)
+
   ncdump(wrf)
 
   atr(wrf,0,verbose = T)
@@ -8,8 +14,8 @@ test_that("utility functios are functional!", {
   atr(wrf,'XLAT','units')
   atr(wrf,0,'DT', action = 'get')
 
-  atr(wrf,'XLAT','units',value = "degree north",verbose = T,action = 'write')
-  atr(wrf,0,'DT',value = 45,verbose = T,action = 'write')
+  atr(copy,'XLAT','units',value = "degree north",verbose = T,action = 'write')
+  atr(copy,0,'DT',value = 45,verbose = T,action = 'write')
 
   vars(wrf,action = 'print')
   v <- vars(wrf)
